/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.01.17
 */
import java.io.*;
import java.util.*;

class Pair<F, S> {
    public F first;
    public S second;
    public Pair(F f, S s) {
        this.first = f;
        this.second = s;
    }
}

class Dsu {
    private int[] p;
    private int[] rank;

    public Dsu(int n) {
        p = new int[n];
        rank = new int[n];
        for (int i = 0; i < n; ++i) p[i] = i;
    }

    public int root(int v) {
        if (v == p[v])
            return v;
        p[v] = root(p[v]);
        return p[v];
    }

    public boolean join(int x, int y) {
        x = root(x);
        y = root(y);
        if (x == y)
            return false;
        if (rank[x] < rank[y]) {
            int tmp = x;
            x = y;
            y = tmp;
        }
        p[y] = x;
        if (rank[x] == rank[y]) ++rank[x];
        return true;
    }
}

class Test {
    private ArrayList<ArrayList<Integer> > g;
    private Pair<Integer, Integer>[] bounds;
    private Edge[] edges;
    private int ptr = 0;
    public int n;

    static class Edge {
        public int from;
        public int to;

        public Edge(int from, int to) {
            this.from = from;
            this.to = to;
        }
    }

    public Test(int n) {
        this.n = n;
        edges = new Edge[n - 1];
        bounds = new Pair[n];
        g = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            g.add(new ArrayList<>());
    }

    public Edge[] getEdges() {
        if (ptr != g.size() - 1)
            throw new RuntimeException("Incorrect structure of tree!");
        return edges;
    }

    public int getNumVertexes() {
        return g.size();
    }

    public void addEdge(int from, int to) {
        edges[ptr++] = new Edge(from, to);
        g.get(from).add(to);
        g.get(to).add(from);
    }

    public ArrayList<Integer> getAdjacent(int v) {
        return g.get(v);
    }

    public void setBounds(int v, int l, int r) {
        bounds[v] = new Pair<>(l, r);
    }

    public Pair<Integer, Integer>[] getBounds() {
        return bounds;
    }
}

public class TestGen {

    private static final boolean FOR_POLYGON = true;
    private static final boolean GEN_DESCR = false;
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0x183718);

    public static final int MAX_N = 100_000;//10^5
    public static final int GROUP1_N = 15;
    public static final int GROUP2_N = 2000;
    public static final int GROUP3_N = MAX_N;
    public static final int MAX_VAL = 10000;

    private String folder;
    private String subtask;
    private int groupNo;
    boolean isSamples = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void setGroupNo(int groupNo) {
        this.groupNo = groupNo;
    }

    void print(Test test) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));
        PrintWriter out = new PrintWriter(path);

        System.out.println(testCount);
        Test.Edge[] ed = test.getEdges();
        if (ed.length != test.getNumVertexes() - 1)
            throw new RuntimeException("Wrong number of edges!");
        ArrayList<Test.Edge> edges = new ArrayList<>();
        if (!isSamples)
            test = coloringTree(test);

        Collections.addAll(edges, ed);
        if (!isSamples) {
            for (Test.Edge e : edges)
                if (random.nextBoolean()) {
                    int tmp = e.from;
                    e.from = e.to;
                    e.to = tmp;
                }
        } else
            for (Test.Edge e : edges) {
                e.from++;
                e.to++;
            }
        int n = test.getNumVertexes();
        out.println(n);
        for (int i = 0; i < edges.size(); ++i)
            if (i + 1 == edges.size())
                out.println(edges.get(i).from + " " + edges.get(i).to);
            else
                out.println(edges.get(i).from + " " + edges.get(i).to);
        Pair<Integer, Integer>[] bounds = test.getBounds();
        for (int i = 0; i < n; ++i)
            out.println(bounds[i].first + " " + bounds[i].second);
        out.close();

        if (GEN_DESCR)
            testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, ""));
    }

    public void genSamples() throws IOException {
        isSamples = true;
        setFolder("tests");
        setSubtask("subtask0");

        System.out.println("==== Generating samples ===");
        Test test1 = new Test(6);
        test1.addEdge(0, 1);
        test1.addEdge(0, 2);
        test1.addEdge(0, 3);
        test1.addEdge(3, 4);
        test1.addEdge(4, 5);
        test1.setBounds(0, 1, 3);
        test1.setBounds(1, 0, 4);
        test1.setBounds(2, 0, 4);
        test1.setBounds(3, 2, 3);
        test1.setBounds(4, 2, 2);
        test1.setBounds(5, 0, 0);
        print(test1);

        Test test2 = new Test(4);
        test2.addEdge(0, 1);
        test2.addEdge(0, 2);
        test2.addEdge(0, 3);
        test2.setBounds(0, 1, 1);
        test2.setBounds(1, 1, 1);
        test2.setBounds(2, 1, 1);
        test2.setBounds(3, 1, 1);
        print(test2);

        isSamples = false;
    }

    public Test coloringTree(Test test) {
        int n = test.getNumVertexes();
        List<Integer> perm = new ArrayList<>();
        for (int i = 1; i < n; ++i)
            perm.add(i);
        Collections.shuffle(perm, random);
        perm.add(0, 0);

        int[] inv = new int[n];
        for (int i = 0; i < perm.size(); ++i)
            inv[perm.get(i)] = i + 1;

        Test.Edge[] edges = test.getEdges();
        for (Test.Edge e : edges) {
            e.from = inv[e.from];
            e.to = inv[e.to];
        }
        Pair<Integer, Integer>[] bounds = test.getBounds();
        Pair<Integer, Integer>[] cp = Arrays.copyOf(bounds, bounds.length);
        for (int i = 0; i < n; ++i) {
            test.setBounds(i, cp[inv[i] - 1].first, cp[inv[i] - 1].second);
        }
        return test;
    }

    public Test genBinaryStructure(int n) {
        Test test = new Test(n);
        int i;
        for (i = 0; 2 * i + 2 < n; ++i) {
            test.addEdge(i, 2 * i + 1);
            test.addEdge(i, 2 * i + 2);
        }
        i = 2 * i + 1;
        for (; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genRandomWithBigDiameter(int n) {
        Test test = new Test(n);
        Dsu dsu = new Dsu(n);
        int edges = 0;
        while (edges < n - 1) {
            int x = random.nextInt(n);
            int y = random.nextInt(n);
            if (dsu.join(x, y)) {
                ++edges;
                test.addEdge(x, y);
            }
        }
        return test;
    }

    public Test genRandomWithManyListsStructure(int n) {
        Test test = new Test(n);
        for (int i = 1; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genLongTreeStructure(int n, int ridgeLen) {
        Test test = new Test(n);
        for (int i = 0; i + 1 < ridgeLen; ++i)
            test.addEdge(i, i + 1);
        for (int i = ridgeLen; i < n; ++i)
            test.addEdge(random.nextInt(i), i);
        return test;
    }

    public Test genNBranchesStructure(int n, int branches) {
        if (n - 1 < branches)
            throw new RuntimeException("Too many branches!");
        Test test = new Test(n);
        int len = (n - 1) / branches;
        int v = 1;
        for (int i = 0; i < branches && v < n; ++i) {
            test.addEdge(0, v++);
            for (int j = 1; j < len && v < n; ++j, ++v)
                test.addEdge(v - 1, v);
        }
        if (v < n) {
            test.addEdge(v, len);
            ++v;
            while (v < n)
                test.addEdge(v - 1, v++);
        }
        return test;
    }

    public Test genBoneStructure(int middle, int left, int right) {
        Test test = new Test(middle + left + right);
        for (int i = 0; i + 1 < middle; ++i)
            test.addEdge(i, i + 1);
        for (int i = middle; i < middle + left; ++i)
            test.addEdge(0, i);
        for (int i = middle + left; i < middle + left + right; ++i)
            test.addEdge(middle - 1, i);
        return test;
    }

    void genHandleTests() throws IOException {
        Test test1 = new Test(2);
        test1.addEdge(0, 1);
        test1.setBounds(0, 1, 1);
        test1.setBounds(1, 0, 1);
        print(test1);

        Test test2 = new Test(4);
        test2.addEdge(0, 1);
        test2.addEdge(0, 2);
        test2.addEdge(0, 3);
        test2.setBounds(0, 0, 1);
        test2.setBounds(1, 0, 1);
        test2.setBounds(2, 0, 0);
        test2.setBounds(3, 0, 0);
        print(test2);
    }

    public void genSubtask(int n, int groupNo, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");
        setGroupNo(groupNo);
        System.out.println("==== Generating group #" + groupNo + " ===");
        int sq = (int)Math.sqrt(n);

        if (groupNo == 1) {
            genHandleTests();

            print(genRandWithLenOne(1, 1, genLongTreeStructure(n, n / 2)));

            print(genRandWithLenOne(2, 1, genLongTreeStructure(n, n / 2)));
            print(genRandWithLenOne(2, 8, genLongTreeStructure(n, n / 3)));

            print(genRandWithLenOne(2, 1, genRandomWithManyListsStructure(n)));
            print(genRandWithLenOne(2, 2, genRandomWithManyListsStructure(n / 2)));

            print(genRandWithLenOne(2, 1, genRandomWithBigDiameter(n)));
            print(genRandWithLenOne(2, 1, genRandomWithBigDiameter(n)));

            print(genRandWithLenOne(2, 16, genBinaryStructure(n)));
            print(genRandWithLenOne(2, 1, genBinaryStructure(n)));

            print(genRandWithLenOne(1, 1, genNBranchesStructure(n, 3)));
            print(genRandWithLenOne(2, 1, genNBranchesStructure(n, sq)));
        } else {
            print(genRandWithLen(MAX_VAL, 50, genLongTreeStructure(n, n / 2)));
            print(genRandWithLen(100, 50, genLongTreeStructure(n, n / 3)));

            print(genRandWithLen(MAX_VAL / 10, 1, genRandomWithManyListsStructure(n)));
            print(genRandWithLen(MAX_VAL / 10, MAX_N, genRandomWithManyListsStructure(sq)));
            print(genRandWithLen(MAX_VAL / 10, 10, genRandomWithManyListsStructure(n / 2)));

            print(genRandWithLen(10, MAX_N, genRandomWithBigDiameter(n)));
            print(genRandWithLen(500, 1, genRandomWithBigDiameter(n)));
            print(genRandWithLen(500, 1000, genRandomWithBigDiameter(n)));

            print(genRandWithLen(MAX_VAL, MAX_N, genBinaryStructure(n)));
            print(genRandWithLen(100, 10, genBinaryStructure(n)));

            print(genRandWithLen(MAX_VAL, MAX_N, genNBranchesStructure(n, 3)));
            print(genRandWithLen(MAX_VAL / 10, MAX_N, genNBranchesStructure(n, sq)));
        }

        if (GEN_DESCR) {
            testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d\"", subtaskCount, n));
            testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
        }
    }

    private Test genRandWithLen(int maxLen, int diff, Test test) {
        return genRandWithLenCommon(MAX_VAL, maxLen, diff, test);
    }

    private Test genRandWithLenOne(int maxLen, int diff, Test test) {
        return genRandWithLenCommon(1, maxLen, diff, test);
    }

    private Test genRandWithLenCommon(int maxVal, int maxLen, int diff, Test test) {
        --maxLen;
        Pair<Integer, Integer>[] bounds = new Pair[diff];
        for (int i = 0; i < diff; ++i) {
            int len = randInt(maxLen + 1);
            int r = randInt(maxVal - len + 1) + len;
            bounds[i] = new Pair<>(r - len, r);
        }

        int n = test.getNumVertexes();
        for (int i = 0; i < n; ++i) {
            int id = randInt(diff);
            test.setBounds(i, bounds[id].first, bounds[id].second);
        }
        return test;
    }


    private int randInt(int n) {
        return random.nextInt(n);
    }

    public void run() throws IOException {
        //new File("../preliminary").mkdir();
        if (!FOR_POLYGON)
            new File("../tests").mkdir();

        if (GEN_DESCR)
            testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(GROUP1_N, 1, 21);
        genSubtask(GROUP2_N, 2, 31);
        genSubtask(GROUP3_N, 3, 48);

        if (GEN_DESCR)
            testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
